
# q-expansion of newform 8512.2.a.j, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8512
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [-1, -1], [1, -2], [1, 0], [-4, -1], [-1, 0], [0, -3], [-1, 0], [3, 0], [4, 1], [-4, 3], [1, 6], [2, -1], [-2, 0], [5, -10], [7, -5], [-9, 6], [-3, 6], [-8, 9], [-1, -4], [9, -3], [10, 0], [-3, -3], [12, -6], [-5, 12], [-7, -4], [-5, 6], [-5, 4], [-9, 12], [-9, 3], [-16, 6], [2, 5], [10, 4], [-17, 6], [-11, 16], [-16, 9], [6, -3], [-11, -3], [1, 4], [4, 10], [-2, -5], [16, -9], [-5, 7], [4, 9], [9, -15], [-7, -6], [-1, 9], [19, -12], [-20, 1], [2, -12], [-8, 13], [-11, 4], [-14, 6], [8, 11], [15, -21], [9, 9], [-9, 9], [-10, 15], [14, -6], [1, 16], [3, -9], [9, -6], [-8, -9], [-8, -5], [-6, -6], [-18, 12], [-7, -9], [2, -15], [-15, -9], [-15, 15], [10, 1], [20, -19], [13, 0], [25, -15], [21, 6], [17, 2], [-4, 11], [16, -30], [-8, 1], [-7, -3], [-27, -6], [25, 6], [10, -20], [13, -30], [-8, 0], [-13, -1], [-19, -7], [19, -9], [-31, 5], [-27, 6], [7, -29], [-26, 25], [-17, -12], [-8, 4], [5, 3], [-8, 16], [-30, 0], [-31, 2], [21, 12], [-6, 12], [7, -15], [14, -7], [-11, 10], [-11, 22], [-5, 18], [19, -3], [34, -8], [-15, 0], [1, -5], [-9, 9], [29, -18], [3, 27], [10, -23], [19, 9], [5, 6], [-24, 27], [-41, 12], [1, -26], [5, -22], [16, -17], [2, -24], [-9, 15], [23, -19], [-31, -4], [-22, -6], [40, 4], [-15, -6], [4, 16], [-43, 0], [8, -24], [15, -12], [-5, 16], [-27, 3], [2, 12], [35, -10], [-4, -24], [14, 14], [8, -12], [-30, 27], [31, -38], [-22, 36], [24, -12], [2, -18], [17, 8], [-32, 24], [-4, 20], [-11, 9], [-32, 37], [30, 9], [-19, -1], [-28, -18], [27, -39], [-37, 24], [9, 12], [-15, 18], [14, 8], [41, -6], [35, -7], [-20, 21], [-26, -20], [19, -11], [-20, 19], [22, 3], [-23, 10], [-10, -10], [31, -26], [30, 18], [29, -3], [-17, 18], [-36, 12], [31, -47], [-28, 9], [34, 7], [37, -24], [16, 18], [28, 16], [23, -15], [-1, 2], [-9, 0], [14, -33], [-10, 27], [15, -48], [5, -3], [-1, 41], [-11, -2], [-10, -4], [25, 18], [9, 3], [-5, -21], [-31, 17], [12, 0], [28, 1], [2, -12], [5, -22], [-46, -4], [13, -23], [11, 0], [-23, -6], [41, 8], [22, 28], [32, -28], [19, -27], [-19, 6], [-26, 18], [-4, -25], [-20, 19], [18, 24], [-12, 18], [34, -38], [1, 3], [-26, 6], [-1, 11], [24, -30], [-51, 18], [-26, 4], [-39, 0], [39, -6], [26, 20], [28, -8], [-23, 34], [3, 12], [-3, 9], [34, -32], [35, 12], [2, 17], [23, -21], [-1, -34], [15, 27], [8, -48], [4, 37], [1, -15], [20, 6], [-15, 6], [7, -2], [17, -45], [31, -53], [-44, 27], [26, -28], [37, 16], [-40, -7], [-12, -30], [43, 0], [-11, -18], [48, 0], [-74, 1], [2, 5], [20, -18], [53, -1], [54, -12], [-55, 8], [-21, -6], [-4, 41], [46, -29], [-32, -15], [-60, -12], [-3, 0], [-55, 6], [-22, 30], [-1, -19], [3, -6], [-20, 24], [-14, 16], [-31, 18], [2, -42], [4, 28], [58, 6], [27, 12], [-39, 0], [32, -24], [11, -13], [8, 15], [44, -18], [-37, -27], [-23, -9], [-35, 39], [44, 12], [-10, 29], [-6, -12], [24, 6], [21, -12], [61, -23], [-47, 12], [16, -32], [12, 18], [62, -30], [-24, 0], [-25, 18], [23, -40], [35, 0], [-53, 52], [43, -8], [-53, 7], [-72, 21], [-6, -36], [-19, -9], [-59, 7], [-39, 42], [-10, -16], [8, -1], [17, 27], [47, 6], [5, 32], [32, -30], [-21, -27], [4, 33], [-5, -12], [-17, -20], [-31, 33], [20, -4], [-22, -39], [25, 10], [46, -47], [9, 18], [-11, 33], [34, -20], [8, -12], [5, 5], [-57, 12], [-5, 3], [-18, -12], [-39, 60], [66, -39], [-8, -5], [-43, -21], [38, -16], [-29, -15], [-45, 51], [-22, -3], [-55, -19], [-29, 7], [-8, 6], [-60, 24], [20, 15], [25, -68], [-23, -18], [46, -41], [9, 36], [-56, 7], [-5, 36], [43, -9], [4, -45], [32, -7], [-37, 44], [21, -30], [67, 10], [10, -5], [34, -36], [46, -54], [-43, 20], [-5, 19], [5, 0], [39, -12], [7, 43], [-20, 51], [-28, -33], [-42, 45], [34, -14], [-9, -18], [-16, -4], [-8, 19], [-59, 48], [-9, 15], [37, -38], [-2, 46], [-3, -36], [32, 12], [39, 0], [18, -51], [-40, 72], [-22, 5], [74, -18], [20, -4], [16, 10], [15, 33], [23, 3], [-20, -50], [-57, -6], [26, -48], [1, 34], [83, -18], [11, 8], [-20, -14], [41, -24], [9, -60], [-18, 36], [-48, 0], [28, -45], [27, -33], [-69, 12], [-63, -9], [-74, 27], [77, -19], [-61, 68], [-6, -45], [-15, 6], [25, -42], [81, -24], [-12, 24], [-50, 60], [27, -27], [-5, 36], [47, -70], [13, -6], [-49, 26], [4, 7], [1, 39], [58, 15], [30, -48], [38, 0], [44, -31], [-89, -6], [-69, 30], [-48, 21], [43, -33], [-18, 30], [26, -55], [3, -30], [-19, 15], [-19, 62], [3, -3], [45, -18], [-51, -6], [15, -12], [61, -29], [12, 9], [-25, -4], [16, -23], [31, -50], [35, -66], [0, -6]]
